#include "Geom.h"

#include "FileUtils.h"

using namespace FCG;

Geom::Geom()
{
}

int FCG::Geom::addVertex( EPSG::Vec3<double> vertex, EPSG::Vec3<double> color )
{
	_vertex.push_back( vertex );
	_color.push_back( color );

	return (int)_vertex.size() - 1;
}

void FCG::Geom::addTriangle( int v1, int v2, int v3 )
{
	_triangle.push_back( EPSG::Vec3<int>( v1, v2, v3 ) );
}

void FCG::Geom::addTriangle( EPSG::Vec3<int> triangle )
{
	_triangle.push_back( triangle );
}

int FCG::Geom::countVertex()
{
	return (int)_vertex.size();
}

int FCG::Geom::countTriangle()
{
	return (int)_triangle.size();
}

const EPSG::Vec3<double>& FCG::Geom::getVertex( int index )
{
	if( index < 0 || index > (int)_vertex.size() )
		return _vertex[0];

	return _vertex[index];
}

const EPSG::Vec3<int>& FCG::Geom::getTriangle( int index )
{
	if( index < 0 || index > (int)_triangle.size() )
		return _triangle[0];

	return _triangle[index];
}

Geom* FCG::Geom::subdivide()
{
	Geom* res = new Geom();

	for( int i = 0; i < (int)_triangle.size(); i++ )
	{
		EPSG::Vec3<double> v1, v2, v3;
		v1 = _vertex[_triangle[i].x];
		v2 = _vertex[_triangle[i].y];
		v3 = _vertex[_triangle[i].z];

		EPSG::Vec3<double> c1, c2, c3;
		c1 = _color[_triangle[i].x];
		c2 = _color[_triangle[i].y];
		c3 = _color[_triangle[i].z];

		EPSG::Vec3<double> v1_2, v1_3, v2_3;
		v1_2 = ( v1 + v2 ) / 2;
		v1_3 = ( v1 + v3 ) / 2;
		v2_3 = ( v2 + v3 ) / 2;

		EPSG::Vec3<double> c1_2, c1_3, c2_3;
		c1_2 = ( c1 + c2 ) / 2;
		c1_3 = ( c1 + c3 ) / 2;
		c2_3 = ( c2 + c3 ) / 2;

		res->addVertex( v1, c1 );		// ( i * 6 ) + 0
		res->addVertex( v2, c2 );		// ( i * 6 ) + 1
		res->addVertex( v3, c3 );		// ( i * 6 ) + 2

		res->addVertex( v1_2, c1_2 );		// ( i * 6 ) + 3
		res->addVertex( v1_3, c1_3 );		// ( i * 6 ) + 4
		res->addVertex( v2_3, c2_3 );		// ( i * 6 ) + 5

		res->addTriangle( ( i * 6 ) + 0, ( i * 6 ) + 3, ( i * 6 ) + 4 );
		res->addTriangle( ( i * 6 ) + 3, ( i * 6 ) + 1, ( i * 6 ) + 5 );
		res->addTriangle( ( i * 6 ) + 4, ( i * 6 ) + 5, ( i * 6 ) + 2 );
		res->addTriangle( ( i * 6 ) + 3, ( i * 6 ) + 5, ( i * 6 ) + 4 );
	}

	return res;
}

void FCG::Geom::print( const char* file )
{
	FILE* out = FileUtils::openFile( file, "wt" );

	int vCount = (int)_vertex.size();
	int tCount = (int)_triangle.size();

	fprintf( out, "%d %d\n", vCount, tCount );

	for( int i = 0; i < vCount; i++ )
		fprintf( out, "%lf %lf %lf	%lf %lf %lf\n", 
					_vertex[i].x, _vertex[i].y, _vertex[i].z, 
					_color[i].x, _color[i].y, _color[i].z );

	for( int i = 0; i < tCount; i++ )
		fprintf( out, "%d %d %d\n", _triangle[i].x, _triangle[i].y, _triangle[i].z );

	fclose( out );
}

void FCG::Geom::printAsMesh( const char* file )
{
	FILE* out = FileUtils::openFile( file, "wt" );

	int tCount = (int)_triangle.size();

	for( int i = 0; i < tCount; i++ )
	{
		fprintf( out, " %lf %lf %lf ", _vertex[_triangle[i].x].x, _vertex[_triangle[i].x].y, _vertex[_triangle[i].x].z );
		fprintf( out, " %lf %lf %lf ", _vertex[_triangle[i].y].x, _vertex[_triangle[i].y].y, _vertex[_triangle[i].y].z );
		fprintf( out, " %lf %lf %lf ", _vertex[_triangle[i].z].x, _vertex[_triangle[i].z].y, _vertex[_triangle[i].z].z );
		fprintf( out, "	" );
		fprintf( out, " %lf %lf %lf ", _color[_triangle[i].x].x, _color[_triangle[i].x].y, _color[_triangle[i].x].z );
		fprintf( out, " %lf %lf %lf ", _color[_triangle[i].y].x, _color[_triangle[i].y].y, _color[_triangle[i].y].z );
		fprintf( out, " %lf %lf %lf ", _color[_triangle[i].z].x, _color[_triangle[i].z].y, _color[_triangle[i].z].z );
		fprintf( out, "\n" );
	}

	fclose( out );
}